/*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
*
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

/* ============================================================================ */
/*                       	     Includes                             		    */
/* ============================================================================ */
#include "pruss_intc_mapping.h"

#include "sel_app_cnfg.h"
#include "am437x.h"
#include "edma.h"
#include "dma.h"
#include "startup.h"
#include "pruicss.h"
#include "osdrv_pruss.h"
#include "board_support.h"
#include "board_gpio.h"
#include "hw_icss.h"
#include "hw_cm_per.h"
#include "hw_edma_tpcc.h"
#include "pruss_register.h"

//PRU-ICSS firmware files
#include "I2C_SMBus_PRU.h"

/* ========================================================================== */
/*                           	Macros & Typedefs                             */
/* ========================================================================== */

/** \brief Global Structure pointer holding PRUSS0 memory Map. */
PRUICSS_Handle pruIcss0Handle;
PRUICSS_Handle pruIcss1Handle;

typedef enum SMBUS_FORMAT
{
	no_command,
	quick_command,
	send_byte,
	receive_byte,
	write_byte,
	write_word,
	read_byte,
	read_word,
	write_block,
	read_block,
} SMBus_Format_t;

/* ========================================================================== */
/*                             		  Defines								  */
/* ========================================================================== */
#define PR1_HOST_EVT_6     					(1)
#define EDMA3_LINK_CHA_NUM					(2)

#define SYS_INT_PRUSS0_EVTOUT0				(191-32)

#define SLAVE_ADDR_TUSB8041					0x44

/* AM437x specific mux bit defines moved from from PRU_SMBus.gel file */
#define AM437X_PIN_OUTPUT               	(0)
#define AM437X_PIN_INPUT					((1 << 5) | (1 << 3))
#define AM437X_PULL_UP						(1 << 17)

/* Defines for PRU memory interface */
#define PRU1_DRAM0							0x54442000
#define PRU_I2C_IRQSTS_RAW					0x24			//I2C Status Raw Register / bit ARDY = 2, bit NACK = 1
#define PRU_I2C_IRQSTS						0x28			//I2C Status Register / bit ARDY = 2, bit NACK = 1
#define PRU_I2C_IRQEN_SET					0x2C			//I2C Interrupt Enable Set Register / bit ARDY = 2, bit NACK = 1
#define PRU_I2C_CNT							0x98			//Data Count Register
#define PRU_I2C_CON							0xA4			//I2C Configuration Register --- 9th bit TRX used as Rd/Wr direction
#define PRU_I2C_SA							0xAC			//I2C Slave Address Register
#define SDA_PINMMUX_VALUE_OUTPUT			0xE0			//SDA pinmux value for input
#define SDA_PINMMUX_VALUE_INPUT				0xE4			//SDA pinmux value for output
#define CLOCK_HALF_CYCLE_DELAY_VALUE		0xE8			//Contains delay value for half clock cycle
#define COMMAND_CODE						0xEC			//Contains Command Code Information
#define SELECTED_SMBUS_COMMAND				0xF0			//Contains Information that which SMBus Command is selected
#define TX_DATA_LOCATION					0x100			//Location where TX data is located
#define RX_DATA_LOCATION					0x200			//Location where RX data is stored

/* Defines for values used in ARM C code */
#define Read_set							0x00000200		//Value written in 9th bit of PRU_I2C_CON register
#define Write_set							0x00000000		//Value written in 9th bit of PRU_I2C_CON register
#define PRU_ICSS_INTC_SRSR0					0x200			//Value for PRU_ICSS_INTC_SRSR0 from TRM
#define PRU_ICSS_INTC_HIPIR2				0x908			//Value for PRU_ICSS_INTC_HIPIR2 from TRM
#define PRU_ICSS_INTC_SISR					0x20			//Value for PRU_ICSS_INTC_SISR from TRM
#define PRU_ICSS_INTC_SICR					0x24			//Value for PRU_ICSS_INTC_SICR from TRM
#define PRU_SYS_EVT_18_SRSR0				0x00040000		//Value written in PRU_ICSS_INTC_SRSR0 register to clear event 18
#define SET_I2C_IRQEN_ARDY_NACK				0x6				//Value written in PRU_I2C_IRQSTS_RAW register's bit position 1 and 2
#define FREQ_VAL_FOR_100KHZ					0x3E4			//Half clock delay value to generate clock of 100KHz


/* ========================================================================== */
/*                       Global variables and constants						  */
/* ========================================================================== */
Task_Handle tsk;

PRUICSS_IrqArgs* pruHandlerArgs;
PRUICSS_IrqArgs* pruHandlerArgs1;

PRUICSS_IntcInitData pruss0_intc_initdata = PRUSS0_PRU_INTC_INITDATA;
PRUICSS_IntcInitData pruss1_intc_initdata = PRUSS1_PRU_INTC_INITDATA;

volatile unsigned char global_interrupt_flag;
uint8_t tx_data_array[256];
uint8_t rx_data_array[256];
uint8_t *tx_data_ptr = tx_data_array;
uint8_t *rx_data_ptr = rx_data_array;

/* brief EDMA default configuration */
static const EDMAChConfig_t EDMA_CH_DEFAULT = { 0U, /* regionId */
0U, /* paRamEntry */
0U, /* queueNum */
FALSE, /* enableEvt */
FALSE /* enableIntr */
}; /* EDMAChConfig_t */

/* ========================================================================== */
/*                               Prototypes									  */
/* ========================================================================== */
void tsk_execute_pru(UArg arg0, UArg arg1);
void edma_config();
void SetEdmaPaRamEntry();
void EDMAModuleClkConfig(void);

void I2c_init();
int I2c_write(SMBus_Format_t smbus_format, uint8_t slave_addr, uint8_t command_code, uint8_t byte_count, uint8_t *tx_data_ptr);
int I2c_read(SMBus_Format_t smbus_format, uint8_t slave_addr, uint8_t command_code, uint8_t *rx_data_ptr);
void PRU2ARM_Interrupt_Handler();

/*
 *  ======== main ========
 */
int main()
{

	Task_Params taskParams;

	BOARDInit(NULL);
	SDKMMUInit(applMmuEntries); // needed first

	// PRU ICSS0
	pruIcss0Handle = (PRUICSS_Handle) malloc(sizeof(PRUICSS_Config));
	pruIcss0Handle->object = (PRUICSS_V1_Object*) malloc(sizeof(PRUICSS_V1_Object));
	pruIcss0Handle->hwAttrs = (PRUICSS_HwAttrs*) malloc(sizeof(PRUICSS_HwAttrs));
	((PRUICSS_V1_Object*) pruIcss0Handle->object)->instance = 0;

	// PRU ICSS1
	pruIcss1Handle = (PRUICSS_Handle) malloc(sizeof(PRUICSS_Config));
	pruIcss1Handle->object = (PRUICSS_V1_Object*) malloc(sizeof(PRUICSS_V1_Object));
	pruIcss1Handle->hwAttrs = (PRUICSS_HwAttrs*) malloc(sizeof(PRUICSS_HwAttrs));
	((PRUICSS_V1_Object*) pruIcss1Handle->object)->instance = 1;

	Board_pinMuxConfig(am437xIdkMux);

	PRUSSDRVInit(pruIcss0Handle);
	PRUSSDRVInit(pruIcss1Handle);

	PRUICSS_enableOCPMasterAccess(pruIcss0Handle);
	PRUICSS_enableOCPMasterAccess(pruIcss1Handle);

	Task_Params_init(&taskParams);
	taskParams.priority = 1;
	tsk = Task_create(tsk_execute_pru, &taskParams, NULL); // this task controls reset

	PRUICSS_pinMuxConfig(pruIcss0Handle, 0x0); // PRUSS pinmuxing
	PRUICSS_pinMuxConfig(pruIcss1Handle, 0x0); // PRUSS pinmuxing

	/* initialize PRUSS INTC with configured profile*/
	PRUICSS_pruIntcInit(pruIcss0Handle, &pruss0_intc_initdata);
	PRUICSS_pruIntcInit(pruIcss1Handle, &pruss1_intc_initdata);

	BIOS_start();

	return 0;
}

void tsk_execute_pru(UArg arg0, UArg arg1)
{
	int status;
	SMBus_Format_t smbus_format;
	uint8_t slave_addr;
	uint8_t command_code;
	uint8_t byte_count;

	//====================================================	For Testing Purpose	=====================================================
	uint8_t tx_byte_count;
	uint8_t rx_byte_count;
	uint8_t temp_data_read_write_byte = 0x5A;
	uint8_t temp_data_read_write_word_byte0 = 0x57;
	uint8_t temp_data_read_write_word_byte1 = 0xA5;
	uint8_t temp_data_read_write_block_byte0 = 0x75;
	uint8_t temp_data_read_write_block_byte1 = 0xD3;
	uint8_t temp_data_read_write_block_byte2 = 0xD7;
	uint8_t temp_data_read_write_block_byte3 = 0x53;

	int quick_cmd_sent = 0;
	int quick_cmd_ACKs = 0;
	int quick_cmd_NACKs = 0;

	int write_byte_sent = 0;
	int write_byte_ACKs = 0;
	int write_byte_NACKs = 0;
	int read_byte_sent = 0;
	int read_byte_ACKs = 0;
	int read_byte_NACKs = 0;
	int read_write_byte_correct_data = 0;
	int read_write_byte_incorrect_data = 0;

	int write_word_sent = 0;
	int write_word_ACKs = 0;
	int write_word_NACKs = 0;
	int read_word_sent = 0;
	int read_word_ACKs = 0;
	int read_word_NACKs = 0;
	int read_write_word_correct_data = 0;
	int read_write_word_incorrect_data = 0;

	int write_block_sent = 0;
	int write_block_ACKs = 0;
	int write_block_NACKs = 0;
	int read_block_sent = 0;
	int read_block_ACKs = 0;
	int read_block_NACKs = 0;
	int same_tx_rx_byte_count = 0;
	int different_tx_rx_byte_count = 0;
	int read_write_block_correct_data = 0;
	int read_write_block_incorrect_data = 0;
	//==================================================================================================================================

	PRUICSS_pruDisable(pruIcss0Handle, 1);

	PRUICSS_pruReset(pruIcss0Handle, 1);

	PRUICSS_setPRUBuffer(pruIcss0Handle, 1, (UInt32*)I2C_SMBus_PRU, sizeof(I2C_SMBus_PRU));

	// LCD_PCLK = pr0_pru1_gpo8 (dss_pclk - mode 5) ---- LED0-RED
	HW_WR_REG32(0x44e108E8, AM437X_PIN_OUTPUT | 5);
	// LCD_AC_BIAS_EN = pr0_pru1_gpo9  (dss_ac-bias_en - mode 5) ---- LED0-GREEN
	HW_WR_REG32(0x44e108EC, AM437X_PIN_OUTPUT | 5);
	// LCD_VSYNC = pr0_pru1_gpo6  (dss_vsync - mode 5) ---- LED0-YELLOW
	HW_WR_REG32(0x44e108E0, AM437X_PIN_OUTPUT | 5);

	// CAM1_DATA3 = pr0_pru1_gpo11 (ccdc1_data3 - mode 4)  SDA
	HW_WR_REG32(0x44e109F4, 0x08060005);

	// CAM1_DATA5 = pr0_pru1_gpo13 (cam1_data5 - mode 4)   SCL
	HW_WR_REG32(0x44e109FC, 0x08020004);

	edma_config();

    //Initialize I2C/SmBus
    I2c_init();

    //Run firmware
    PRUICSS_pruExecProgram(pruIcss0Handle, 1);

    // wait for firmware to signal ready
    do
    {
    	;
    }while(!(HW_RD_REG32(PRU1_DRAM0 + PRU_I2C_IRQSTS_RAW) & 0x4));

    //Write 0x50 at register offset 0x05 in TUSB8041 device to enable writing to its serial number registers
    smbus_format = write_byte;
    slave_addr = 0x44;
    command_code = 0x05;
    byte_count = 0;
    tx_data_array[0] = 0x50;
    status = I2c_write(smbus_format, slave_addr, command_code, byte_count, tx_data_ptr);

    //Application Code (In this case testing application)
	do
	{
		// Testing for Quick_Command
		smbus_format = quick_command;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0;
		byte_count = 0;
		status = I2c_write(smbus_format, slave_addr, command_code, byte_count, tx_data_ptr);
		quick_cmd_sent++;
		if (status == 0)
			quick_cmd_ACKs++;
		else
			quick_cmd_NACKs++;

		// Testing for Write_Byte and Read_Byte
		smbus_format = write_byte;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0x30;
		byte_count = 0;
		tx_data_array[0] = temp_data_read_write_byte;
		status = I2c_write(smbus_format, slave_addr, command_code, byte_count, tx_data_ptr);
		write_byte_sent++;
		if (status == 0)
			write_byte_ACKs++;
		else
			write_byte_NACKs++;
		smbus_format = read_byte;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0x30;
		status = I2c_read(smbus_format, slave_addr, command_code, rx_data_ptr);
		read_byte_sent++;
		if (status == 0)
			read_byte_ACKs++;
		else
			read_byte_NACKs++;

		if (tx_data_array[0] == rx_data_array[0])
			read_write_byte_correct_data++;
		else
			read_write_byte_incorrect_data++;

		// Testing for Write_Word and Read_Word
		smbus_format = write_word;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0x30;
		byte_count = 0;
		tx_data_array[0] = temp_data_read_write_word_byte0;
		tx_data_array[1] = temp_data_read_write_word_byte1;
		status = I2c_write(smbus_format, slave_addr, command_code, byte_count, tx_data_ptr);
		write_word_sent++;
		if (status == 0)
			write_word_ACKs++;
		else
			write_word_NACKs++;
		smbus_format = read_word;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0x30;
		status = I2c_read(smbus_format, slave_addr, command_code, rx_data_ptr);
		read_word_sent++;
		if (status == 0)
			read_word_ACKs++;
		else
			read_word_NACKs++;

		if (tx_data_array[0] == rx_data_array[0])
			read_write_word_correct_data++;
		else
			read_write_word_incorrect_data++;

		if (tx_data_array[1] == rx_data_array[1])
			read_write_word_correct_data++;
		else
			read_write_word_incorrect_data++;

		// Testing for Write_Block and Read_Block
		smbus_format = write_block;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0x30;
		byte_count = 4;
		tx_data_array[0] = temp_data_read_write_block_byte0;
		tx_data_array[1] = temp_data_read_write_block_byte1;
		tx_data_array[2] = temp_data_read_write_block_byte2;
		tx_data_array[3] = temp_data_read_write_block_byte3;
		status = I2c_write(smbus_format, slave_addr, command_code, byte_count, tx_data_ptr);
		write_block_sent++;
		tx_byte_count = HW_RD_REG8((PRU1_DRAM0 + PRU_I2C_CNT));
		if (status == 0)
			write_block_ACKs++;
		else
			write_block_NACKs++;
		smbus_format = read_block;
		slave_addr = SLAVE_ADDR_TUSB8041;
		command_code = 0x30;
		status = I2c_read(smbus_format, slave_addr, command_code, rx_data_ptr);
		read_block_sent++;
		rx_byte_count = HW_RD_REG8((PRU1_DRAM0 + PRU_I2C_CNT));
		if (status == 0)
			read_block_ACKs++;
		else
			read_block_NACKs++;

		if (tx_byte_count == rx_byte_count)
			same_tx_rx_byte_count++;
		else
			different_tx_rx_byte_count++;

		if (tx_data_array[0] == rx_data_array[0])
			read_write_block_correct_data++;
		else
			read_write_block_incorrect_data++;

		if (tx_data_array[1] == rx_data_array[1])
			read_write_block_correct_data++;
		else
			read_write_block_incorrect_data++;

		if (tx_data_array[2] == rx_data_array[2])
			read_write_block_correct_data++;
		else
			read_write_block_incorrect_data++;

		if (tx_data_array[3] == rx_data_array[3])
			read_write_block_correct_data++;
		else
			read_write_block_incorrect_data++;

		temp_data_read_write_byte++;
		temp_data_read_write_word_byte0++;
		temp_data_read_write_word_byte1++;
		temp_data_read_write_block_byte0++;
		temp_data_read_write_block_byte1++;
		temp_data_read_write_block_byte2++;
		temp_data_read_write_block_byte3++;
	}while(1);
}

void edma_config()
{
	EDMAChConfig_t edmaChnl = EDMA_CH_DEFAULT;

	edmaChnl.region = 0;
	edmaChnl.enableIntr = FALSE;
	edmaChnl.paramIdx = 1;
	edmaChnl.queueNum = 1;
	edmaChnl.enableEvt = TRUE;

	// clear PRU1 data memory0
	memset((void *) 0x54442000, 0, (4 * 1024));

	/* Configuring the system clocks for EDMA. */
	EDMAModuleClkConfig();

	/* (a) Initiate the channel */
	/* Initialization of EDMA3 */
	EDMAInit(SOC_EDMA3CC_REG, 0);

	/* (b) Channel mapping */
	/* program the DCHMAP with the parameter set number */
	EDMAChConfig(SOC_EDMA3CC_REG, EDMA_CH_TYPE_DMA, 1, &edmaChnl);

	/* Step 2 - Parameter set setup */
	SetEdmaPaRamEntry();

	/* Step 4 - Initiate transfer */
	/* Enable EDMA Transfer */
	EDMATransferStart(SOC_EDMA3CC_REG, 0, 1, DMA_XFER_TRIGGER_TYPE_EVENT);
}

void SetEdmaPaRamEntry()
{
	EDMAParamEntry_t paramSet;
	unsigned char *p = (unsigned char *) &paramSet;
	unsigned int index = 0;

	/* Clean-up the contents of structure variable. */
	for (index = 0; index < sizeof(paramSet); index++) {
		p[index] = 0;
	}

	/* Write the SDA output pinmux value to PRU1 DRAM0 */
	HW_WR_REG32((PRU1_DRAM0 + SDA_PINMMUX_VALUE_OUTPUT), 0x08010004);
	/* Write the SDA input pinmux value to PRU1 DRAM0 */
	/* CTRL_CONF_CAM1_DATA3 -- > Receiver enabled(18) ; PU/PD enabled(16) ; PU selected(17); mode(0-3) set to input */
	HW_WR_REG32((PRU1_DRAM0 + SDA_PINMMUX_VALUE_INPUT), 0x08060005);

	/* Fill the PaRAM Set with transfer specific information */
	/* PaRAM set 1 */
	/* OPT = 0x0 */
	paramSet.word1 = 0x0;
	/* SRC = ICSS0 Data RAM0 */
	//paramSet.word2 = 0x54442000;
	paramSet.word2 = 0x544420E0;
	/* BCNT = 2 , ACNT = 4 */
	paramSet.word3 = (1 << 17) | (1 << 2);
	/* DST = 0x44E109F4 pinmux register */
	paramSet.word4 = 0x44E109F4;
	/* DSTBIDX = 0x4 ; SRCBIDX = 0x4 */
	paramSet.word5 = (0 << 18) | (1 << 2);
	/* LINK address = 0x4040 */
	paramSet.word6 = (unsigned short) (EDMA_TPCC_PARAM(EDMA3_LINK_CHA_NUM));
	/* DSTCIDX = 0x0 ; SRCCIDX = 0x0 */
	paramSet.word7 = 0;
	/* CCNT = 0x1 */
	paramSet.word8 = 1;

	/* Now write the PaRAM Set */
	EDMASetParamEntry(SOC_EDMA3CC_REG, 1, &paramSet);

	/* Now write the linked PaRAM Set */
	EDMASetParamEntry(SOC_EDMA3CC_REG, EDMA3_LINK_CHA_NUM, &paramSet);
}

/*
 ** This function enables the system L3 clocks.
 ** This also enables the clocks for EDMA instance.
 */
void EDMAModuleClkConfig(void)
{
	/* Configuring L3 Interface Clocks. */

	/* Writing to MODULEMODE field of CM_PER_L3_CLKCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_L3_CLKCTRL),
	PRCM_CM_PER_L3_CLKCTRL_MODULEMODE_ENABLE);

	/* Writing to MODULEMODE field of CM_PER_L3_INSTR_CLKCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_L3_INSTR_CLKCTRL),
	PRCM_CM_PER_L3_INSTR_CLKCTRL_MODULEMODE_ENABLE);

	/* Writing to CLKTRCTRL field of CM_PER_L3_CLKSTCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_L3_CLKSTCTRL),
	PRCM_CM_PER_L3_CLKSTCTRL_CLKTRCTRL_SW_WKUP);

	/* Writing to CLKTRCTRL field of CM_PER_L3S_CLKSTCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_L3S_CLKSTCTRL),
	PRCM_CM_PER_L3S_CLKSTCTRL_CLKTRCTRL_SW_WKUP);

	/* Configuring clocks for EDMA3 TPCC and TPTCs. */

	/* Writing to MODULEMODE field of CM_PER_TPCC_CLKCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_TPCC_CLKCTRL),
	PRCM_CM_PER_TPCC_CLKCTRL_MODULEMODE_ENABLE);

	/* Writing to MODULEMODE field of CM_PER_TPTC0_CLKCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_TPTC0_CLKCTRL),
	PRCM_CM_PER_TPTC0_CLKCTRL_MODULEMODE_ENABLE);

	/* Writing to MODULEMODE field of CM_PER_TPTC1_CLKCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_TPTC1_CLKCTRL),
	PRCM_CM_PER_TPTC1_CLKCTRL_MODULEMODE_ENABLE);

	/* Writing to MODULEMODE field of CM_PER_TPTC2_CLKCTRL register. */
	HW_WR_REG32((SOC_CM_PER_REG + PRCM_CM_PER_TPTC2_CLKCTRL),
	PRCM_CM_PER_TPTC2_CLKCTRL_MODULEMODE_ENABLE);

	/* DMA in non-idle mode */
	HW_WR_REG32(0x49800010, 0x00000028);
	HW_WR_REG32(0x49900010, 0x00000028);
	HW_WR_REG32(0x49a00010, 0x00000028);
}

/*
 ** ISR for event 17 from PRU to ARM
 ** Sets the flag to let main know that there was an event
 ** Reads the event number received from PRU and clears it
 */
void PRU2ARM_Interrupt_Handler()
{
	uint32_t temp32;
	global_interrupt_flag = 1;
	temp32 = HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + PRU_ICSS_INTC_HIPIR2);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + PRU_ICSS_INTC_SICR, temp32);
}

/*
 ** Initializes the I2C/SMBus link by setting the frequency
 ** It also enables the interrupt for ARDY and NACK
 */
void I2c_init()
{
	HW_WR_REG32((PRU1_DRAM0 + CLOCK_HALF_CYCLE_DELAY_VALUE), FREQ_VAL_FOR_100KHZ);
	HW_WR_REG32((PRU1_DRAM0 + PRU_I2C_IRQEN_SET), SET_I2C_IRQEN_ARDY_NACK);
}

/*
 ** Used for I2C/SMBus write formats
 */
int I2c_write(SMBus_Format_t smbus_format, uint8_t slave_addr, uint8_t command_code, uint8_t byte_count, uint8_t *tx_data_ptr)
{
	int i;
	uint32_t temp32;

	// If ARDY not set yet by PRU for letting ARM access its register interface then return error code
	temp32 = HW_RD_REG32((PRU1_DRAM0 + PRU_I2C_IRQSTS_RAW));
	if (((temp32 >> 2) & 1) != 1)
	{
		return -2;
	}

	// Configuring I2C/SMBus shared memory interface
	HW_WR_REG32((PRU1_DRAM0 + I2C_SA), slave_addr);
	HW_WR_REG32((PRU1_DRAM0 + I2C_CON), Write_set);
	HW_WR_REG32((PRU1_DRAM0 + COMMAND_CODE), command_code);
	HW_WR_REG32((PRU1_DRAM0 + SELECTED_SMBUS_COMMAND), smbus_format);

	// Copy the data to be transmitted from the tx_data_array into PRU register interface according to the right I2C/SMBus format
	switch(smbus_format)
	{
		case send_byte :
			HW_WR_REG8((PRU1_DRAM0 + TX_DATA_LOCATION), *tx_data_ptr);
			break;
		case write_byte :
			HW_WR_REG8((PRU1_DRAM0 + TX_DATA_LOCATION), *tx_data_ptr);
			break;
		case write_word :
			HW_WR_REG8((PRU1_DRAM0 + TX_DATA_LOCATION), *tx_data_ptr);
			HW_WR_REG8((PRU1_DRAM0 + TX_DATA_LOCATION + 1), *(tx_data_ptr + 1));
			break;
		case write_block :
			HW_WR_REG8((PRU1_DRAM0 + PRU_I2C_CNT), byte_count);
			for (i = 0; i < byte_count; i++)
			{
				HW_WR_REG8((PRU1_DRAM0 + TX_DATA_LOCATION + i), *(tx_data_ptr + i));
			}
			break;
		default :
			break;
	}

	// Clear global interrupt flag so that next interrupt can be serviced
	global_interrupt_flag = 0;

	// Send event to PRU to execute current I2C/SMBus format
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + PRU_ICSS_INTC_SRSR0, PRU_SYS_EVT_18_SRSR0);

	// Wait for PRU event (i.e. flag is 1)
	while(global_interrupt_flag == 0)
	{
		;
	}

	// Read out status register
	temp32 = HW_RD_REG32((PRU1_DRAM0 + PRU_I2C_IRQSTS_RAW));

	// If ACK OKAY, return 0, otherwise error code -1
	if (((temp32 >> 1) & 1) == 1)
	{
		return -1;
	}
	else
	{
		return 0;
	}
}

/*
 ** Used for I2C/SMBus read formats
 */
int I2c_read(SMBus_Format_t smbus_format, uint8_t slave_addr, uint8_t command_code, uint8_t *rx_data_ptr)
{
	int i;
	uint8_t byte_count;
	uint32_t temp32;

	// If ARDY not set yet by PRU for letting ARM access its register interface then return error code
	temp32 = HW_RD_REG32((PRU1_DRAM0 + PRU_I2C_IRQSTS_RAW));
	if (((temp32 >> 2) & 1) != 1)
	{
		return -2;
	}

	// Configuring I2C/SMBus shared memory interface
	HW_WR_REG32((PRU1_DRAM0 + I2C_SA), slave_addr);
	HW_WR_REG32((PRU1_DRAM0 + I2C_CON), Read_set);
	HW_WR_REG32((PRU1_DRAM0 + COMMAND_CODE), command_code);
	HW_WR_REG32((PRU1_DRAM0 + SELECTED_SMBUS_COMMAND), smbus_format);

	// Clear global interrupt flag so that next interrupt can be serviced
	global_interrupt_flag = 0;

	// Send event to PRU to execute current I2C/SMBus format
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + PRU_ICSS_INTC_SRSR0, PRU_SYS_EVT_18_SRSR0);

	// Wait for PRU event (i.e. flag is 1)
	while(global_interrupt_flag == 0)
	{
		;
	}

	// Read out status register
	temp32 = HW_RD_REG32((PRU1_DRAM0 + PRU_I2C_IRQSTS_RAW));

	// if ACK OKAY, return 0, otherwise error code -1
	if (((temp32 >> 1) & 1) == 1)
	{
		return -1;
	}
	else
	{
		// Copy the received data in the rx_data_array from PRU register interface according to the right I2C/SMBus format
		switch(smbus_format)
		{
			case receive_byte :
				*rx_data_ptr = HW_RD_REG8((PRU1_DRAM0 + RX_DATA_LOCATION));
				break;
			case read_byte :
				*rx_data_ptr = HW_RD_REG8((PRU1_DRAM0 + RX_DATA_LOCATION));
				break;
			case read_word :
				*rx_data_ptr = HW_RD_REG8((PRU1_DRAM0 + RX_DATA_LOCATION));
				*(rx_data_ptr + 1) = HW_RD_REG8((PRU1_DRAM0 + RX_DATA_LOCATION + 1));
				break;
			case read_block :
				byte_count = HW_RD_REG8((PRU1_DRAM0 + PRU_I2C_CNT));
				for (i = 0; i < byte_count; i++)
				{
					*(rx_data_ptr + i) = HW_RD_REG8((PRU1_DRAM0 + RX_DATA_LOCATION + i));
				}
				break;
			default :
				break;
		}
		return 0;
	}
}
